using System.Reflection;
using GraphicsManager;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.GUI.MainScreen.Interfaces;
using Luski.GUI.Windows;
using Luski.net.Enums;
using Luski.net.JsonTypes;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.MainScreen.UI;

public class Friend : UserControl, IChannelPick
{
    public SocketRemoteUser User;

    public SocketTextChannel Channel
    {
        get
        {
            return User.Channel!;
        }
    }
    public Rectangle r, rr;
    public Label Username, Status;
    public Friend(SocketRemoteUser person)
    {
        User = person;
        Size = new((int)(240 * Globals.Settings.Scale), (int)(62* Globals.Settings.Scale));
        BackgroundColor = new(34, 34, 34, 255);
        
        Controls.Add(Username = new Label() { Font = Globals.DefaultFont, Text = person.Username, Location = new((int)(58 * Globals.Settings.Scale),(int)(14 * Globals.Settings.Scale))});
        Controls.Add(Status = new Label() { Scale = 0.8f, Font = Globals.DefaultFont, Text = person.Status.ToString(), Location = new((int)(58 * Globals.Settings.Scale),(int)(34* Globals.Settings.Scale))});
        
        
        Controls.Add(this.rr=new Rectangle(Globals.ms.TextureManager.AddTexture(Tools.GetResourceBytes(Assembly.GetExecutingAssembly(),
            "Luski.Resources.Textures.Status.png"))) { Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            Location = new((int)(10 * Globals.Settings.Scale), (int)(10 * Globals.Settings.Scale)),
            Size = new ((int)(42 * Globals.Settings.Scale))
        });
        Username.Location = new(Username.Location.X,
            (this.rr.Location.Y + (this.rr.Size.Y / 2) - ((Username.TrueHeight + 5 + Status.TrueHeight) / 2) -
             (Username.Size.Y - (Username.TrueHeight)))
        );
        Status.Location = new(Username.Location.X, (int)(Username.Location.Y + Username.Font.PixelHeight + 5 - (Status.Size.Y - Status.TrueHeight)));
        this.rr.BackgroundColor = person.Status switch
        {
            UserStatus.Online => new(115, 255, 111, 255),
            UserStatus.Idle => new(255, 229, 84, 255),
            UserStatus.DoNotDisturb => new(255, 51, 79, 255),
            UserStatus.Offline or UserStatus.Invisible or _ => Color4.LightGray
        };
        r = new Rectangle(this.rr.Textures.First()) { Location = new(this.rr.Location.X + 4,this.rr.Location.Y + 4), Size = new(this.rr.Size.X - 8)};
        Texture t = Globals.ms.TextureManager.AddTexture(person.GetAvatar(CancellationToken.None).Result);
        t.Unit = TextureUnit.Texture1;
        r.Textures.Add(t);
        r.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
        Controls.Add(r);
        this.Clicked += AllOnClicked;
        ContextMenu = new((int)(196 * Globals.Settings.Scale));
        RoundedButton rrr, rr2;
        ContextMenu.Items.Add(rrr=new()
        {
            InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan,
            Size = new((int)(32 * Globals.Settings.Scale)), Font = Globals.DefaultFont, Text = "Export Keys"
        });
        ContextMenu.Items.Add(rr2=new()
        {
            InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan,
            Size = new((int)(32 * Globals.Settings.Scale)), Font = Globals.DefaultFont, Text = "Open In New Window"
        });
        rr2.Clicked += Rr2OnClicked;
        rrr.Clicked += RrOnClicked;
    }

    private Task Rr2OnClicked(IRenderObject arg)
    {
        GLFW.WindowHint(WindowHintBool.Decorated, false);
        ChatWindow w = new(this);
        
        //w.WindowBorder = WindowBorder.Resizable;
       // ContextMenu!.HideContext(Window!);
        //Window!.DrawFrame();
        w.StartRender();
        //w.IsEventDriven = true;
        //Window.Windows.Add(w);
        //w.StartRenderAsync();
        //w.StartRender();
        //ContextMenu!.HideContext(Window!);
        return Task.CompletedTask;
    }

  

    private Task RrOnClicked(IRenderObject arg)
    {
        _ = User.Channel!.SendKeysToUsers(CancellationToken.None);
        ContextMenu!.HideContext(Window!);
        return Task.CompletedTask;
    }

    private Task AllOnClicked(IRenderObject arg)
    {
        if (ClickCon is not null) _ = ClickCon.Invoke(this);
        return Task.CompletedTask;
    }

    public event Func<IChannelPick, Task>? ClickCon;
}