using System.Reflection;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.StartPage.UI;

public class CreateAccount : UserControl
{
    private RoundedButton button;
    private Textbox Password, Email, Username;
    private Rectangle rec;
    private string? pfp;
    
    public event Func<Task>? ChangeToApp;
    
    public CreateAccount()
    {
        Size = new((int)(312 * Globals.Settings.Scale), (int)(545 * Globals.Settings.Scale));
        Rectangle tt;
        Controls.Add(tt=new Rectangle(Globals.LuskiTexture) { Size = new((int)(179*Globals.Settings.Scale), (int)(189*Globals.Settings.Scale))});
        Label t;
        Controls.Add(t=new Label() { Scale = 1.6f * (float)Globals.Settings.Scale, Font = Globals.DefaultFont, Location = new((int)(199*Globals.Settings.Scale)), Text = "Luski", Color = new(243, 119, 53, 255) });
        t.Location = new((Size.X / 2) - (t.Size.X / 2), t.Location.Y);
        tt.Location = new((Size.X / 2) - (tt.Size.X / 2), tt.Location.Y);
        Controls.Add(Email = new Textbox() { TextLocation = TextLocation.TrueCenterLeft, WatermarkFont = Globals.DefaultFont, WatermarkText = "Email", Font = Globals.DefaultFont,  Location = new((int)(27 * Globals.Settings.Scale),(int)(280 * Globals.Settings.Scale)), Size = new((int)(261 * Globals.Settings.Scale),(int)(27 * Globals.Settings.Scale)), InsideColor = new(28,28,28,255), BorderColor = Color4.DarkCyan });
        Controls.Add(Password = new Textbox() { TextLocation = TextLocation.TrueCenterLeft, WatermarkFont = Globals.DefaultFont, WatermarkText = "Password", Font = Globals.DefaultFont, PasswordChar = '●', Location = new(Email.Location.X,(int)(346 * Globals.Settings.Scale)), Size = new(Email.Size.X, Email.Location.X), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(Username = new Textbox() { TextLocation = TextLocation.TrueCenterLeft, WatermarkFont = Globals.DefaultFont, WatermarkText = "Username", Font = Globals.DefaultFont, Location = new(Email.Location.X,(int)(400 * Globals.Settings.Scale)), Size = new((int)(196 * Globals.Settings.Scale), Email.Location.X), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(button = new() { Font = Globals.DefaultFont, Text = "Create Account", Location = new(Email.Location.X, (int)(456 * Globals.Settings.Scale)), Size = new(Email.Size.X, (int)(46 * Globals.Settings.Scale)), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(rec = new Rectangle(){ Location = new((int)(228 * Globals.Settings.Scale), (int)(384 * Globals.Settings.Scale)), Size = new((int)(50 * Globals.Settings.Scale)), BackgroundColor = Color4.Red});
        Password.KeyPress += PasswordOnKeyPress;
        Email.KeyPress += EmailOnKeyPress;
        button.Clicked += ButtonOnClicked;
        Username.KeyPress += UsernameOnKeyPress;
        rec.FilesDroped += RecOnFilesDroped;
    }

    private Task UsernameOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Password.Focus(); return Task.CompletedTask; } 
        if (arg.Key == Keys.Enter || arg.Key == Keys.KeyPadEnter) ButtonOnClicked(Username);
        return Task.CompletedTask;
    }

    private Task RecOnFilesDroped(string[] arg)
    {
        if (!arg[0].ToLower().EndsWith("png")) return Task.CompletedTask;
        
        pfp = arg[0];
        if (!rec.Textures.Any())
        {
            Controls.Remove(rec);
            Vector2i oldl = rec.Location, olds = rec.Size;
            rec = new Rectangle(Globals.ms.TextureManager.AddTexture(Tools.GetResourceBytes(Assembly.GetExecutingAssembly(),
                "Luski.Resources.Textures.Status.png"))) { Location = oldl, Size = olds };
            Texture tex = Globals.ms.TextureManager.AddTexture(File.ReadAllBytes(arg[0]));
            tex.Unit = TextureUnit.Texture1;
            rec.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
            rec.Textures.Add(tex);
            Controls.Add(rec);
            //rec = new(Globals.ms.TextureManager.AddTexture(File.ReadAllBytes(arg[0]), true)){ Location = new(350, 585), Size = new(100) };Controls.Add(rec);
        }
        else
        {
            rec.Textures[1] = Globals.ms.TextureManager.AddTexture(File.ReadAllBytes(arg[0]));
            rec.Textures[1].Unit = TextureUnit.Texture1;
        }
        
        Window!.DrawFrame();
        return Task.CompletedTask;
    }

    private bool tab;
    private Task EmailOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab){ Password.Focus(); tab = true;}
        return Task.CompletedTask;
    }

    private Task PasswordOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (tab)
        {
            tab = false;
            return Task.CompletedTask;
        };
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Email.Focus(); return Task.CompletedTask; }
        if (arg.Key == Keys.Tab ){ Username.Focus(); return Task.CompletedTask; }
        return Task.CompletedTask;
    }

    private Task ButtonOnClicked(IRenderObject arg)
    {
        try
        {
            string[] arr = new string[]
            {
                Email.Text,
                Password.Text,
                Username.Text,
                pfp!
            };
            if (arr.Any(s => string.IsNullOrEmpty(s))) return Task.CompletedTask;
            Globals.Luski = Server.CreateAccount(Email.Text, Password.Text, Username.Text, pfp!, CancellationToken.None, Globals.UpdaterSettings.Branch);
            ChangeToApp!.Invoke();
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }
}