﻿using System.Diagnostics;
using System.Reflection;
using Luski;
using Luski.Clesses;
using Luski.GUI.MainScreen;
using OpenTK.Windowing.Common.Input;
using SixLabors.ImageSharp.PixelFormats;
using Image = OpenTK.Windowing.Common.Input.Image;


try
{
    Globals.Settings = Globals.GetSettings(Path.Combine(Globals.LuskiPath, "Settings.json"), SettingsContext.Default.Settings);
    Globals.UpdaterSettings = Globals.GetSettings(Path.Combine(Globals.LuskiPath, "UpdaterSettings.json"), UpdaterSettingsContext.Default.UpdaterSettings);
    Assembly assembly = Assembly.GetExecutingAssembly();
    Stream? resource_stream = assembly.GetManifestResourceStream($"Luski.Resources.Textures.Luski.png");
    MemoryStream ms = new();
    resource_stream?.CopyTo(ms);
    var Logo = SixLabors.ImageSharp.Image.Load<Rgba32>(ms.ToArray());

    //List<byte> pixels = new List<byte>(4 * Logo.Width * Logo.Height);
    Span<byte> pixels = new byte[4 * Logo.Width * Logo.Height].AsSpan();
    Logo.CopyPixelDataTo(pixels);
   
    Globals.Icon = new WindowIcon(new Image(Logo.Width, Logo.Height, pixels.ToArray()));
    Globals.ms = new MainScreen();
    Globals.ms.StartRender();
    Globals.ms.Dispose();
}
catch (Exception ex)
{
    Console.WriteLine(ex);
}

if (Globals.Download)
{
    Console.WriteLine("Auto Update Starting");
    List<string> arguments = new List<string>
    {
        "--process",
        Process.GetCurrentProcess().ProcessName,
        "--remotedirectory",
        "Luski",
        "--localdirectory",
        AppDomain.CurrentDomain.BaseDirectory,
        "--branch",
        Globals.UpdaterSettings.Branch.ToString(),
        "--selfcontained",
        Globals.UpdaterSettings.SelfContained.ToString().ToLower(),
        "--platform",
        Globals.UpdaterSettings.Platform,
        "--setconfig",
        Path.Combine(Globals.LuskiPath, "UpdaterSettings.json")
    };
    if (Globals.UpdaterSettings.AutoLaunch)
    {
        arguments.Add("--dll");
        arguments.Add(AppDomain.CurrentDomain.FriendlyName);
    }
    Process p = new();
    p.StartInfo.FileName = Globals.UpdaterSettings.Updater;
    p.StartInfo.WorkingDirectory = new FileInfo(Globals.UpdaterSettings.Updater!).Directory!.FullName;
    p.StartInfo.Arguments = $"\"{string.Join("\" \"", arguments)}\"";
    p.Start();
    Environment.Exit(0);
}